import sys
import numpy as np
from scipy import sparse
import matplotlib.pyplot as plt
from sklearn import svm
from sklearn import tree
from sklearn import linear_model
from sklearn.decomposition import PCA
from sklearn.grid_search import GridSearchCV
from nltk.tokenize import TweetTokenizer
import nltk
import os
import time

import testimplfeatures as tif
import zcombineblocks

t_start = time.time()

testBlockInd = 1
zcombineblocks.makeData(testBlockInd)

sarcfname = "bigsarcdata.txt"
sarcposfname = "bigsarcposdata.txt"
regfname = "bigregdata.txt"
regposfname = "bigregposdata.txt"

useManualData = False

if(useManualData):
##    testsarcfname = "testrilsarcdata.txt"
##    testsarcposfname = "testrilsarcposdata.txt"
##    testregfname = "testrilregdata.txt"
##    testregposfname = "testrilregposdata.txt"
##    testsarcfname = "testrilsarcdatahts.txt"
##    testsarcposfname = "testrilsarcposdatahts.txt"
##    testregfname = "testrilregdatahts.txt"
##    testregposfname = "testrilregposdatahts.txt"

    testsarcfname = "testmysarcdata.txt"
    testsarcposfname = "testmysarcposdata.txt"
    testregfname = "testmyregdata.txt"
    testregposfname = "testmyregposdata.txt"
##    testsarcfname = "testmysarcdatahts.txt"
##    testsarcposfname = "testmysarcposdatahts.txt"
##    testregfname = "testmyregdatahts.txt"
##    testregposfname = "testmyregposdatahts.txt"
else:
    testsarcfname = "testbigsarcdata.txt"
    testsarcposfname = "testbigsarcposdata.txt"
    testregfname = "testbigregdata.txt"
    testregposfname = "testbigregposdata.txt"

tknzr = TweetTokenizer()

useUnigrams = True
usePos = True
usePunct = True
useLssd = True
useIncon = True
useWvsE = True
useVvsR = True
useSent = True

skewLim = 1.5
cutOffWordFreq = 200
wutlimreg = 80001
wutlimsarc = 20001

Y = []
corrans = []
wordlist = {}
tweets = []
postags = []
testtweets = []
testpostags = []

param_grid = [
  {'C': [0.1, 1, 10, 100], 'kernel': ['linear']},
  {'C': [0.1, 1, 10, 100], 'gamma': [0.01, 0.001, 0.0001], 'kernel': ['rbf']},
 ]

#WORDLIST FOR UNIGRAM STUFF START
wut = 0
file = open(regfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    for t in tokens:
        if t in wordlist:
            wordlist[t][0] += 1
        else:
            wordlist[t] = [1,0]
    if wut > wutlimreg:
        break
    wut += 1
file.close()

wut = 0
file = open(sarcfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    for t in tokens:
        if t in wordlist:
            wordlist[t][1] += 1
        else:
            wordlist[t] = [0,1]
    if wut > wutlimsarc:
        break
    wut += 1
file.close()

wordsToBeDeleted = []
for word in wordlist:
    if (wordlist[word][0] + wordlist[word][1]) < cutOffWordFreq:
        wordsToBeDeleted.append(word)

for word in wordsToBeDeleted:
    del wordlist[word]

if("#sarcasm" in wordlist):
    del wordlist["#sarcasm"]

if("#sarcastic" in wordlist):
    del wordlist["#sarcastic"]

wordsToBeDeleted = []
for word in wordlist:
    wordlist[word][1] *= 4
    if wordlist[word][0] == 0 or wordlist[word][1] == 0:
        wordsToBeDeleted.append(word)
        continue
    if ("0" in word) or ("1" in word) or ("2" in word) or ("3" in word) or ("4" in word) or ("5" in word) or ("6" in word) or ("7" in word) or ("8" in word) or ("9" in word):
        wordsToBeDeleted.append(word)
        continue
    skew = max(wordlist[word][1]/wordlist[word][0], wordlist[word][0]/wordlist[word][1])
    if skew < skewLim:
        wordsToBeDeleted.append(word)

for word in wordsToBeDeleted:
    del wordlist[word]

counter = 0
for word in wordlist:
    wordlist[word] = counter
    counter += 1
    
print(len(wordlist))
#WORDLIST FOR UNIGRAM STUFF END

#READ AND TOKENIZE TWEETS START
wut = 0
file = open(regfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    tweets.append(tokens)
    Y.append(False)
    if wut > wutlimreg:
        break
    wut += 1
file.close()

wut = 0
file = open(sarcfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    tweets.append(tokens)
    Y.append(True)
    if wut > wutlimsarc:
        break
    wut += 1
file.close()

file = open(testregfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    testtweets.append(tokens)
    corrans.append(False)
file.close()

file = open(testsarcfname, "r", encoding="utf-8")
for tweet in file:
    tweet = tweet.strip()
    tokens = tknzr.tokenize(tweet)
    testtweets.append(tokens)
    corrans.append(True)
file.close()
#READ AND TOKENIZE TWEETS STOP

#READ POS TAGS START
wut = 0
file = open(regposfname, "r", encoding="utf-8")
for line in file:
    line = line.strip()
    line = line.split("\t")
    for i in range(len(line)):
        line[i] = line[i].split()
    postags.append(line)
    if wut > wutlimreg:
        break
    wut += 1
file.close()

wut = 0
file = open(sarcposfname, "r", encoding="utf-8")
for line in file:
    line = line.strip()
    line = line.split("\t")
    for i in range(len(line)):
        line[i] = line[i].split()
    postags.append(line)
    if wut > wutlimsarc:
        break
    wut += 1
file.close()

file = open(testregposfname, "r", encoding="utf-8")
for line in file:
    line = line.strip()
    line = line.split("\t")
    for i in range(len(line)):
        line[i] = line[i].split()
    testpostags.append(line)
file.close()

file = open(testsarcposfname, "r", encoding="utf-8")
for line in file:
    line = line.strip()
    line = line.split("\t")
    for i in range(len(line)):
        line[i] = line[i].split()
    testpostags.append(line)
file.close()
#READ POS TAGS STOP

#GENERATE FEATURE MATRIX START
featureMatrix = [[] for i in range(len(tweets))]

if(useUnigrams):
    uniFtMx = [None for i in range(len(tweets))]
    counter = 0
    for tweet in tweets:
        uniFtMx[counter] = [0 for i in range(len(wordlist))]
        for token in tweet:
            if token in wordlist:
                uniFtMx[counter][wordlist[token]] = 1
        counter += 1
##    uniPca = PCA(n_components=50)
##    uniPca.fit(uniFtMx)
##    trUniFtMx = uniPca.transform(uniFtMx)
##    counter = 0
##    for row in trUniFtMx.tolist():
##        featureMatrix[counter].extend(row)
##        counter += 1
    counter = 0
    for row in uniFtMx:
        featureMatrix[counter].extend(row)
        counter += 1

if(usePos):
    posFtMx = [None for i in range(len(tweets))]
    counter = 0
    for line in postags:
        posFtMx[counter] = tif.posFeatureize(line)
        counter += 1
    posPca = PCA(n_components=30)
    posPca.fit(posFtMx)
    trPosFtMx = posPca.transform(posFtMx)
    counter = 0
    for row in trPosFtMx.tolist():
        featureMatrix[counter].extend(row)
        counter += 1
##    counter = 0
##    for row in posFtMx:
##        featureMatrix[counter].extend(row)
##        counter += 1

for i in range(len(tweets)):
    semantic_vals = tif.semanticValues(tweets[i])
    sv_afinn = semantic_vals[0]
    sv_nrcuni = semantic_vals[1]
    sv_nrcbi = semantic_vals[2]
    sv_nrcpair = semantic_vals[3]
    sv_emoti = semantic_vals[4]
    realTwL = tif.realLength(postags[i])

    if(usePunct):
        featureMatrix[i].extend(tif.punctFeatureize(tweets[i], realTwL))
        
    if(useLssd):
        featureMatrix[i].extend(tif.lssdFeatureize(tweets[i], sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))

    if(useIncon):
        featureMatrix[i].extend(tif.inconFeatureize(tweets[i]))
        
    if(useWvsE):
        featureMatrix[i].extend(tif.wordvsemotiFeatureize(sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))
        
    if(useVvsR):
        featureMatrix[i].extend(tif.verbvsrestFeatureize(postags[i], sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))

    if(useSent):
        featureMatrix[i].extend(tif.sentFeatureize(sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))
                                
#GENERATE FEATURE MATRIX STOP

t_pit = time.time()
print(t_pit - t_start)

print(len(featureMatrix))
print(len(featureMatrix[0]))
print(len(testtweets))

#RUN CLASSIFIER START
clf = svm.SVC(C=1.4, kernel='rbf', gamma=0.014, cache_size=500)
#clf = tree.DecisionTreeClassifier(max_depth=15)
#clf = linear_model.LogisticRegression(penalty='l2', C=1.0)

#clf.fit(featureMatrix, Y)
spFtMx = sparse.csr_matrix(featureMatrix)
clf.fit(spFtMx, Y)

##grid_search = GridSearchCV(clf, param_grid=param_grid, scoring='f1')
##grid_search.fit(spFtMx, Y)
##for score in grid_search.grid_scores_:
##    print(score)
##print("\n\n")
##print(grid_search.best_estimator_)
##print(grid_search.best_score_)
##print(grid_search.best_params_)
##
##os.remove("bigsarcdata.txt")
##os.remove("bigsarcposdata.txt")
##os.remove("bigregdata.txt")
##os.remove("bigregposdata.txt")
##os.remove("testbigsarcdata.txt")
##os.remove("testbigsarcposdata.txt")
##os.remove("testbigregdata.txt")
##os.remove("testbigregposdata.txt")

#RUN CLASSIFIER STOP

#TESTING START
true_positive = 0
true_negative = 0
false_positive = 0
false_negative = 0

for i in range(len(testtweets)):
    semantic_vals = tif.semanticValues(testtweets[i])
    sv_afinn = semantic_vals[0]
    sv_nrcuni = semantic_vals[1]
    sv_nrcbi = semantic_vals[2]
    sv_nrcpair = semantic_vals[3]
    sv_emoti = semantic_vals[4]
    realTwL = tif.realLength(testpostags[i])
    
    features = []
    
    if(useUnigrams):
        uniFts = [0 for i in range(len(wordlist))]
        for token in testtweets[i]:
            if token in wordlist:
                uniFts[wordlist[token]] = 1
##        trUniFts = uniPca.transform([uniFts])
##        features.extend(trUniFts.tolist()[0])
        features.extend(uniFts)
    
    if(usePos):
        posFts = tif.posFeatureize(testpostags[i])
        trPosFts = posPca.transform([posFts])
        features.extend(trPosFts.tolist()[0])
##        features.extend(posFts)

    if(usePunct):
        features.extend(tif.punctFeatureize(testtweets[i], realTwL))
        
    if(useLssd):
        features.extend(tif.lssdFeatureize(testtweets[i], sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))

    if(useIncon):
        features.extend(tif.inconFeatureize(testtweets[i]))
        
    if(useWvsE):
        features.extend(tif.wordvsemotiFeatureize(sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))
        
    if(useVvsR):
        features.extend(tif.verbvsrestFeatureize(testpostags[i], sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))

    if(useSent):
        features.extend(tif.sentFeatureize(sv_afinn, sv_nrcuni, sv_nrcbi, sv_nrcpair, sv_emoti))                    

    verdict = clf.predict([features])

    if(corrans[i] == True):
        if(verdict[0] == corrans[i]):
            true_positive += 1
        else:
            false_negative += 1
    else:
        if(verdict[0] == corrans[i]):
            true_negative += 1
        else:
            false_positive += 1

file.close()

precision = true_positive/(true_positive + false_positive)
recall = true_positive/(true_positive + false_negative)
accuracy = (true_positive + true_negative)/(true_positive + true_negative + false_positive + false_negative)
fonescore = (2 * precision * recall)/(precision + recall)
#TESTING STOP

print("\n")
print("tp:   " + str(true_positive))
print("tn:   " + str(true_negative))
print("fp:   " + str(false_positive))
print("fn:   " + str(false_negative))
print("\n")
print("prec:   " + str(precision))
print("rec:   " + str(recall))
print("accu:   " + str(accuracy))
print("fscr:   " + str(fonescore))

##os.remove(sarcfname)
##os.remove(sarcposfname)
##os.remove(regfname)
##os.remove(regposfname)
##os.remove(testsarcfname)
##os.remove(testsarcposfname)
##os.remove(testregfname)
##os.remove(testregposfname)

os.remove("bigsarcdata.txt")
os.remove("bigsarcposdata.txt")
os.remove("bigregdata.txt")
os.remove("bigregposdata.txt")
os.remove("testbigsarcdata.txt")
os.remove("testbigsarcposdata.txt")
os.remove("testbigregdata.txt")
os.remove("testbigregposdata.txt")

t_end = time.time()
print(t_end - t_start)
